@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, AnyStr, Callable, Dict, List, Optional, Tuple
 
 from gradio.blocks import Block
 
@@ -15,6 +15,7 @@ class Changeable(Block):
         inputs: List[Component],
         outputs: List[Component],
         status_tracker: Optional[StatusTracker] = None,
+        api_name: AnyStr = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -26,6 +27,7 @@ class Changeable(Block):
             inputs: List of inputs
             outputs: List of outputs
             status_tracker: StatusTracker to visualize function progress
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of input and outputs components, return should be a list of values for output component.
         Returns: None
         """
@@ -35,6 +37,7 @@ class Changeable(Block):
             inputs,
             outputs,
             status_tracker=status_tracker,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
@@ -49,6 +52,7 @@ class Clickable(Block):
         inputs: List[Component],
         outputs: List[Component],
         status_tracker: Optional[StatusTracker] = None,
+        api_name: AnyStr = None,
         queue=None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -60,6 +64,7 @@ class Clickable(Block):
             inputs: List of inputs
             outputs: List of outputs
             status_tracker: StatusTracker to visualize function progress
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
             _preprocess: If False, will not run preprocessing of component data before running 'fn'.
             _postprocess: If False, will not run postprocessing of component data before returning 'fn' output.
@@ -71,6 +76,7 @@ class Clickable(Block):
             inputs,
             outputs,
             status_tracker=status_tracker,
+            api_name=api_name,
             queue=queue,
             js=_js,
             preprocess=_preprocess,
@@ -85,6 +91,7 @@ class Submittable(Block):
         inputs: List[Component],
         outputs: List[Component],
         status_tracker: Optional[StatusTracker] = None,
+        api_name: AnyStr = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -96,6 +103,7 @@ class Submittable(Block):
             inputs: List of inputs
             outputs: List of outputs
             status_tracker: StatusTracker to visualize function progress
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
         Returns: None
         """
@@ -105,6 +113,7 @@ class Submittable(Block):
             inputs,
             outputs,
             status_tracker=status_tracker,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
@@ -118,6 +127,7 @@ class Editable(Block):
         fn: Callable,
         inputs: List[Component],
         outputs: List[Component],
+        api_name: AnyStr = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -128,6 +138,7 @@ class Editable(Block):
             fn: Callable function
             inputs: List of inputs
             outputs: List of outputs
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
         Returns: None
         """
@@ -136,6 +147,7 @@ class Editable(Block):
             fn,
             inputs,
             outputs,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
@@ -149,6 +161,7 @@ class Clearable(Block):
         fn: Callable,
         inputs: List[Component],
         outputs: List[Component],
+        api_name: AnyStr = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -159,6 +172,7 @@ class Clearable(Block):
             fn: Callable function
             inputs: List of inputs
             outputs: List of outputs
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
         Returns: None
         """
@@ -167,6 +181,7 @@ class Clearable(Block):
             fn,
             inputs,
             outputs,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
@@ -180,6 +195,7 @@ class Playable(Block):
         fn: Callable,
         inputs: List[Component],
         outputs: List[Component],
+        api_name: AnyStr = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -190,6 +206,7 @@ class Playable(Block):
             fn: Callable function
             inputs: List of inputs
             outputs: List of outputs
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
         Returns: None
         """
@@ -198,6 +215,7 @@ class Playable(Block):
             fn,
             inputs,
             outputs,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
@@ -209,6 +227,7 @@ class Playable(Block):
         fn: Callable,
         inputs: List[Component],
         outputs: List[Component],
+        api_name: Optional[AnyStr] = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -219,6 +238,7 @@ class Playable(Block):
             fn: Callable function
             inputs: List of inputs
             outputs: List of outputs
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
         Returns: None
         """
@@ -227,6 +247,7 @@ class Playable(Block):
             fn,
             inputs,
             outputs,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
@@ -238,6 +259,7 @@ class Playable(Block):
         fn: Callable,
         inputs: List[Component],
         outputs: List[Component],
+        api_name: AnyStr = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -248,6 +270,7 @@ class Playable(Block):
             fn: Callable function
             inputs: List of inputs
             outputs: List of outputs
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
         Returns: None
         """
@@ -256,6 +279,7 @@ class Playable(Block):
             fn,
             inputs,
             outputs,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
@@ -269,6 +293,7 @@ class Streamable(Block):
         fn: Callable,
         inputs: List[Component],
         outputs: List[Component],
+        api_name: AnyStr = None,
         queue: Optional[bool] = None,
         _js: Optional[str] = None,
         _preprocess: bool = True,
@@ -279,6 +304,7 @@ class Streamable(Block):
             fn: Callable function
             inputs: List of inputs
             outputs: List of outputs
+            api_name: Defining this parameter exposes the endpoint in the api docs
             _js: Optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
         Returns: None
         """
@@ -288,6 +314,7 @@ class Streamable(Block):
             fn,
             inputs,
             outputs,
+            api_name=api_name,
             js=_js,
             preprocess=_preprocess,
             postprocess=_postprocess,
