@@ -1,13 +1,41 @@
+"""This module should not be used directly as its API is subject to change. Instead,
+use the `gr.Blocks.load()` or `gr.Interface.load()` functions."""
+
 import base64
 import json
 import re
+from copy import deepcopy
+from typing import Callable, Dict
 
 import requests
 
-from gradio import inputs, outputs, utils
+import gradio
+from gradio import components, utils
 
 
-def get_huggingface_interface(model_name, api_key, alias):
+def load_blocks_from_repo(name, src=None, api_key=None, alias=None, **kwargs):
+    """Creates and returns a Blocks instance from several kinds of Hugging Face repos:
+    1) A model repo
+    2) A Spaces repo running Gradio 2.x
+    3) A Spaces repo running Gradio 3.x
+    """
+    if src is None:
+        tokens = name.split(
+            "/"
+        )  # Separate the source (e.g. "huggingface") from the repo name (e.g. "google/vit-base-patch16-224")
+        assert (
+            len(tokens) > 1
+        ), "Either `src` parameter must be provided, or `name` must be formatted as {src}/{repo name}"
+        src = tokens[0]
+        name = "/".join(tokens[1:])
+    assert src.lower() in factory_methods, "parameter: src must be one of {}".format(
+        factory_methods.keys()
+    )
+    blocks: gradio.Blocks = factory_methods[src](name, api_key, alias, **kwargs)
+    return blocks
+
+
+def get_models_interface(model_name, api_key, alias, **kwargs):
     model_url = "https://huggingface.co/{}".format(model_name)
     api_url = "https://api-inference.huggingface.co/models/{}".format(model_name)
     print("Fetching model from: {}".format(model_url))
@@ -49,8 +77,8 @@ def get_huggingface_interface(model_name, api_key, alias):
     pipelines = {
         "audio-classification": {
             # example model: https://hf.co/ehcalabres/wav2vec2-lg-xlsr-en-speech-emotion-recognition
-            "inputs": inputs.Audio(label="Input", source="upload", type="filepath"),
-            "outputs": outputs.Label(label="Class", type="confidences"),
+            "inputs": components.Audio(source="upload", type="filepath", label="Input"),
+            "outputs": components.Label(label="Class"),
             "preprocess": lambda i: base64.b64decode(
                 i["data"].split(",")[1]
             ),  # convert the base64 representation to binary
@@ -60,8 +88,8 @@ def get_huggingface_interface(model_name, api_key, alias):
         },
         "audio-to-audio": {
             # example model: https://hf.co/speechbrain/mtl-mimic-voicebank
-            "inputs": inputs.Audio(label="Input", source="upload", type="filepath"),
-            "outputs": outputs.Audio(label="Output"),
+            "inputs": components.Audio(source="upload", type="filepath", label="Input"),
+            "outputs": components.Audio(label="Output"),
             "preprocess": lambda i: base64.b64decode(
                 i["data"].split(",")[1]
             ),  # convert the base64 representation to binary
@@ -69,8 +97,8 @@ def get_huggingface_interface(model_name, api_key, alias):
         },
         "automatic-speech-recognition": {
             # example model: https://hf.co/jonatasgrosman/wav2vec2-large-xlsr-53-english
-            "inputs": inputs.Audio(label="Input", source="upload", type="filepath"),
-            "outputs": outputs.Textbox(label="Output"),
+            "inputs": components.Audio(source="upload", type="filepath", label="Input"),
+            "outputs": components.Textbox(label="Output"),
             "preprocess": lambda i: base64.b64decode(
                 i["data"].split(",")[1]
             ),  # convert the base64 representation to binary
@@ -78,21 +106,21 @@ def get_huggingface_interface(model_name, api_key, alias):
         },
         "feature-extraction": {
             # example model: hf.co/julien-c/distilbert-feature-extraction
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Dataframe(label="Output"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Dataframe(label="Output"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r.json()[0],
         },
         "fill-mask": {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: {i["token_str"]: i["score"] for i in r.json()},
         },
         "image-classification": {
             # Example: https://huggingface.co/google/vit-base-patch16-224
-            "inputs": inputs.Image(label="Input Image", type="filepath"),
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "inputs": components.Image(type="filepath", label="Input Image"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda i: base64.b64decode(
                 i.split(",")[1]
             ),  # convert the base64 representation to binary
@@ -111,52 +139,55 @@ def get_huggingface_interface(model_name, api_key, alias):
         # TODO: also: support NER pipeline, object detection, table question answering
         "question-answering": {
             "inputs": [
-                inputs.Textbox(label="Context", lines=7),
-                inputs.Textbox(label="Question"),
+                components.Textbox(lines=7, label="Context"),
+                components.Textbox(label="Question"),
+            ],
+            "outputs": [
+                components.Textbox(label="Answer"),
+                components.Label(label="Score"),
             ],
-            "outputs": [outputs.Textbox(label="Answer"), outputs.Label(label="Score")],
             "preprocess": lambda c, q: {"inputs": {"context": c, "question": q}},
             "postprocess": lambda r: (r.json()["answer"], r.json()["score"]),
         },
         "summarization": {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Textbox(label="Summary"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Textbox(label="Summary"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r.json()[0]["summary_text"],
         },
         "text-classification": {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: {
                 i["label"].split(", ")[0]: i["score"] for i in r.json()[0]
             },
         },
         "text-generation": {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Textbox(label="Output"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Textbox(label="Output"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r.json()[0]["generated_text"],
         },
         "text2text-generation": {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Textbox(label="Generated Text"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Textbox(label="Generated Text"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r.json()[0]["generated_text"],
         },
         "translation": {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Textbox(label="Translation"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Textbox(label="Translation"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r.json()[0]["translation_text"],
         },
         "zero-shot-classification": {
             "inputs": [
-                inputs.Textbox(label="Input"),
-                inputs.Textbox(label="Possible class names (" "comma-separated)"),
-                inputs.Checkbox(label="Allow multiple true classes"),
+                components.Textbox(label="Input"),
+                components.Textbox(label="Possible class names (" "comma-separated)"),
+                components.Checkbox(label="Allow multiple true classes"),
             ],
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda i, c, m: {
                 "inputs": i,
                 "parameters": {"candidate_labels": c, "multi_class": m},
@@ -169,16 +200,16 @@ def get_huggingface_interface(model_name, api_key, alias):
         "sentence-similarity": {
             # example model: hf.co/sentence-transformers/distilbert-base-nli-stsb-mean-tokens
             "inputs": [
-                inputs.Textbox(
-                    label="Source Sentence", default="That is a happy person"
+                components.Textbox(
+                    value="That is a happy person", label="Source Sentence"
                 ),
-                inputs.Textbox(
+                components.Textbox(
                     lines=7,
-                    label="Sentences to compare to",
                     placeholder="Separate each sentence by a newline",
+                    label="Sentences to compare to",
                 ),
             ],
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda src, sentences: {
                 "inputs": {
                     "source_sentence": src,
@@ -191,22 +222,22 @@ def get_huggingface_interface(model_name, api_key, alias):
         },
         "text-to-speech": {
             # example model: hf.co/julien-c/ljspeech_tts_train_tacotron2_raw_phn_tacotron_g2p_en_no_space_train
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Audio(label="Audio"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Audio(label="Audio"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": encode_to_base64,
         },
         "text-to-image": {
             # example model: hf.co/osanseviero/BigGAN-deep-128
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Image(label="Output"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Image(label="Output"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": encode_to_base64,
         },
         "token-classification": {
             # example model: hf.co/huggingface-course/bert-finetuned-ner
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.HighlightedText(label="Output"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.HighlightedText(label="Output"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r,  # Handled as a special case in query_huggingface_api()
         },
@@ -253,57 +284,16 @@ def get_huggingface_interface(model_name, api_key, alias):
         "title": model_name,
     }
 
-    return interface_info
+    kwargs = dict(interface_info, **kwargs)
+    interface = gradio.Interface(**kwargs)
+    interface.api_mode = True  # So interface doesn't run pre/postprocess.
+    return interface
 
 
-def load_interface(name, src=None, api_key=None, alias=None):
-    if src is None:
-        tokens = name.split(
-            "/"
-        )  # Separate the source (e.g. "huggingface") from the repo name (e.g. "google/vit-base-patch16-224")
-        assert (
-            len(tokens) > 1
-        ), "Either `src` parameter must be provided, or `name` must be formatted as {src}/{repo name}"
-        src = tokens[0]
-        name = "/".join(tokens[1:])
-    assert src.lower() in repos, "parameter: src must be one of {}".format(repos.keys())
-    interface_info = repos[src](name, api_key, alias)
-    return interface_info
-
-
-def interface_params_from_config(config_dict):
-    # instantiate input component and output component
-    config_dict["inputs"] = [
-        inputs.get_input_instance(component)
-        for component in config_dict["input_components"]
-    ]
-    config_dict["outputs"] = [
-        outputs.get_output_instance(component)
-        for component in config_dict["output_components"]
-    ]
-    parameters = {
-        "allow_flagging",
-        "allow_screenshot",
-        "article",
-        "description",
-        "flagging_options",
-        "inputs",
-        "outputs",
-        "show_input",
-        "show_output",
-        "theme",
-        "title",
-    }
-    config_dict = {k: config_dict[k] for k in parameters}
-    return config_dict
-
-
-def get_spaces_interface(model_name, api_key, alias):
+def get_spaces(model_name, api_key, alias, **kwargs):
     space_url = "https://huggingface.co/spaces/{}".format(model_name)
     print("Fetching interface from: {}".format(space_url))
     iframe_url = "https://hf.space/embed/{}/+".format(model_name)
-    api_url = "https://hf.space/embed/{}/api/predict/".format(model_name)
-    headers = {"Content-Type": "application/json"}
 
     r = requests.get(iframe_url)
     result = re.search(
@@ -313,35 +303,115 @@ def get_spaces_interface(model_name, api_key, alias):
         config = json.loads(result.group(1))
     except AttributeError:
         raise ValueError("Could not load the Space: {}".format(model_name))
-    interface_info = interface_params_from_config(config)
+    if "allow_flagging" in config:  # Create an Interface for Gradio 2.x Spaces
+        return get_spaces_interface(model_name, config, alias, **kwargs)
+    else:  # Create a Blocks for Gradio 3.x Spaces
+        return get_spaces_blocks(model_name, config)
+
+
+def get_spaces_blocks(model_name, config):
+    def streamline_config(config: dict) -> dict:
+        """Streamlines the blocks config dictionary to fix components that don't render correctly."""
+        # TODO(abidlabs): Need a better way to fix relative paths in dataset component
+        for c, component in enumerate(config["components"]):
+            if component["type"] == "dataset":
+                config["components"][c]["props"]["visible"] = False
+        return config
+
+    config = streamline_config(config)
+    api_url = "https://hf.space/embed/{}/api/predict/".format(model_name)
+    headers = {"Content-Type": "application/json"}
+
+    fns = []
+    for d, dependency in enumerate(config["dependencies"]):
+        if dependency["backend_fn"]:
+
+            def get_fn(outputs, fn_index):
+                def fn(*data):
+                    data = json.dumps({"data": data, "fn_index": fn_index})
+                    response = requests.post(api_url, headers=headers, data=data)
+                    result = json.loads(response.content.decode("utf-8"))
+                    try:
+                        output = result["data"]
+                    except KeyError:
+                        raise KeyError(
+                            f"Could not find 'data' key in response from external Space. Response received: {result}"
+                        )
+                    if len(outputs) == 1:
+                        output = output[0]
+                    return output
+
+                return fn
+
+            fn = get_fn(deepcopy(dependency["outputs"]), d)
+            fns.append(fn)
+        else:
+            fns.append(None)
+    return gradio.Blocks.from_config(config, fns)
+
+
+def get_spaces_interface(model_name, config, alias, **kwargs):
+    def streamline_config(config: dict) -> dict:
+        """Streamlines the interface config dictionary to remove unnecessary keys."""
+        config["inputs"] = [
+            components.get_component_instance(component)
+            for component in config["input_components"]
+        ]
+        config["outputs"] = [
+            components.get_component_instance(component)
+            for component in config["output_components"]
+        ]
+        parameters = {
+            "article",
+            "description",
+            "flagging_options",
+            "inputs",
+            "outputs",
+            "theme",
+            "title",
+        }
+        config = {k: config[k] for k in parameters}
+        return config
+
+    config = streamline_config(config)
+    api_url = "https://hf.space/embed/{}/api/predict/".format(model_name)
+    headers = {"Content-Type": "application/json"}
 
     # The function should call the API with preprocessed data
     def fn(*data):
         data = json.dumps({"data": data})
         response = requests.post(api_url, headers=headers, data=data)
         result = json.loads(response.content.decode("utf-8"))
-        output = result["data"]
+        try:
+            output = result["data"]
+        except KeyError:
+            raise KeyError(
+                f"Could not find 'data' key in response from external Space. Response received: {result}"
+            )
         if (
-            len(interface_info["outputs"]) == 1
+            len(config["outputs"]) == 1
         ):  # if the fn is supposed to return a single value, pop it
             output = output[0]
-        if len(interface_info["outputs"]) == 1 and isinstance(
+        if len(config["outputs"]) == 1 and isinstance(
             output, list
         ):  # Needed to support Output.Image() returning bounding boxes as well (TODO: handle different versions of gradio since they have slightly different APIs)
             output = output[0]
         return output
 
     fn.__name__ = alias if (alias is not None) else model_name
-    interface_info["fn"] = fn
+    config["fn"] = fn
 
-    return interface_info
+    kwargs = dict(config, **kwargs)
+    interface = gradio.Interface(**kwargs)
+    interface.api_mode = True  # So interface doesn't run pre/postprocess.
+    return interface
 
 
-repos = {
-    # for each repo, we have a method that returns the Interface given the model name & optionally an api_key
-    "huggingface": get_huggingface_interface,
-    "models": get_huggingface_interface,
-    "spaces": get_spaces_interface,
+factory_methods: Dict[str, Callable] = {
+    # for each repo type, we have a method that returns the Interface given the model name & optionally an api_key
+    "huggingface": get_models_interface,
+    "models": get_models_interface,
+    "spaces": get_spaces,
 }
 
 
@@ -367,8 +437,10 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.AudioClassificationPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Audio(label="Input", source="microphone", type="filepath"),
-            "outputs": outputs.Label(label="Class", type="confidences"),
+            "inputs": components.Audio(
+                source="microphone", type="filepath", label="Input"
+            ),
+            "outputs": components.Label(label="Class"),
             "preprocess": lambda i: {"inputs": i},
             "postprocess": lambda r: {i["label"].split(", ")[0]: i["score"] for i in r},
         }
@@ -376,8 +448,10 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.AutomaticSpeechRecognitionPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Audio(label="Input", source="microphone", type="filepath"),
-            "outputs": outputs.Textbox(label="Output"),
+            "inputs": components.Audio(
+                source="microphone", type="filepath", label="Input"
+            ),
+            "outputs": components.Textbox(label="Output"),
             "preprocess": lambda i: {"inputs": i},
             "postprocess": lambda r: r["text"],
         }
@@ -385,8 +459,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.FeatureExtractionPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Dataframe(label="Output"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Dataframe(label="Output"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r[0],
         }
@@ -394,8 +468,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.FillMaskPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: {i["token_str"]: i["score"] for i in r},
         }
@@ -403,8 +477,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.ImageClassificationPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Image(label="Input Image", type="filepath"),
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "inputs": components.Image(type="filepath", label="Input Image"),
+            "outputs": components.Label(type="confidences", label="Classification"),
             "preprocess": lambda i: {"images": i},
             "postprocess": lambda r: {i["label"].split(", ")[0]: i["score"] for i in r},
         }
@@ -413,10 +487,13 @@ def load_from_pipeline(pipeline):
     ):
         pipeline_info = {
             "inputs": [
-                inputs.Textbox(label="Context", lines=7),
-                inputs.Textbox(label="Question"),
+                components.Textbox(lines=7, label="Context"),
+                components.Textbox(label="Question"),
+            ],
+            "outputs": [
+                components.Textbox(label="Answer"),
+                components.Label(label="Score"),
             ],
-            "outputs": [outputs.Textbox(label="Answer"), outputs.Label(label="Score")],
             "preprocess": lambda c, q: {"context": c, "question": q},
             "postprocess": lambda r: (r["answer"], r["score"]),
         }
@@ -424,8 +501,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.SummarizationPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Textbox(label="Input", lines=7),
-            "outputs": outputs.Textbox(label="Summary"),
+            "inputs": components.Textbox(lines=7, label="Input"),
+            "outputs": components.Textbox(label="Summary"),
             "preprocess": lambda x: {"inputs": x},
             "postprocess": lambda r: r[0]["summary_text"],
         }
@@ -433,8 +510,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.TextClassificationPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda x: [x],
             "postprocess": lambda r: {i["label"].split(", ")[0]: i["score"] for i in r},
         }
@@ -442,8 +519,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.TextGenerationPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Textbox(label="Output"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Textbox(label="Output"),
             "preprocess": lambda x: {"text_inputs": x},
             "postprocess": lambda r: r[0]["generated_text"],
         }
@@ -451,8 +528,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.TranslationPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Textbox(label="Translation"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Textbox(label="Translation"),
             "preprocess": lambda x: [x],
             "postprocess": lambda r: r[0]["translation_text"],
         }
@@ -460,8 +537,8 @@ def load_from_pipeline(pipeline):
         pipeline, transformers.Text2TextGenerationPipeline
     ):
         pipeline_info = {
-            "inputs": inputs.Textbox(label="Input"),
-            "outputs": outputs.Textbox(label="Generated Text"),
+            "inputs": components.Textbox(label="Input"),
+            "outputs": components.Textbox(label="Generated Text"),
             "preprocess": lambda x: [x],
             "postprocess": lambda r: r[0]["generated_text"],
         }
@@ -470,11 +547,11 @@ def load_from_pipeline(pipeline):
     ):
         pipeline_info = {
             "inputs": [
-                inputs.Textbox(label="Input"),
-                inputs.Textbox(label="Possible class names (" "comma-separated)"),
-                inputs.Checkbox(label="Allow multiple true classes"),
+                components.Textbox(label="Input"),
+                components.Textbox(label="Possible class names (" "comma-separated)"),
+                components.Checkbox(label="Allow multiple true classes"),
             ],
-            "outputs": outputs.Label(label="Classification", type="confidences"),
+            "outputs": components.Label(label="Classification"),
             "preprocess": lambda i, c, m: {
                 "sequences": i,
                 "candidate_labels": c,
